// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "GstGLAPI", sizeof(GstGLAPI), alignof(GstGLAPI));
    printf("%s;%zu;%zu\n", "GstGLAllocationParams", sizeof(GstGLAllocationParams), alignof(GstGLAllocationParams));
    printf("%s;%zu;%zu\n", "GstGLAsyncDebug", sizeof(GstGLAsyncDebug), alignof(GstGLAsyncDebug));
    printf("%s;%zu;%zu\n", "GstGLBaseFilter", sizeof(GstGLBaseFilter), alignof(GstGLBaseFilter));
    printf("%s;%zu;%zu\n", "GstGLBaseFilterClass", sizeof(GstGLBaseFilterClass), alignof(GstGLBaseFilterClass));
    printf("%s;%zu;%zu\n", "GstGLBaseMemory", sizeof(GstGLBaseMemory), alignof(GstGLBaseMemory));
    printf("%s;%zu;%zu\n", "GstGLBaseMemoryAllocator", sizeof(GstGLBaseMemoryAllocator), alignof(GstGLBaseMemoryAllocator));
    printf("%s;%zu;%zu\n", "GstGLBaseMemoryAllocatorClass", sizeof(GstGLBaseMemoryAllocatorClass), alignof(GstGLBaseMemoryAllocatorClass));
    printf("%s;%zu;%zu\n", "GstGLBaseMemoryError", sizeof(GstGLBaseMemoryError), alignof(GstGLBaseMemoryError));
    printf("%s;%zu;%zu\n", "GstGLBaseMemoryTransfer", sizeof(GstGLBaseMemoryTransfer), alignof(GstGLBaseMemoryTransfer));
    printf("%s;%zu;%zu\n", "GstGLBaseMixer", sizeof(GstGLBaseMixer), alignof(GstGLBaseMixer));
    printf("%s;%zu;%zu\n", "GstGLBaseMixerClass", sizeof(GstGLBaseMixerClass), alignof(GstGLBaseMixerClass));
    printf("%s;%zu;%zu\n", "GstGLBaseMixerPad", sizeof(GstGLBaseMixerPad), alignof(GstGLBaseMixerPad));
    printf("%s;%zu;%zu\n", "GstGLBaseMixerPadClass", sizeof(GstGLBaseMixerPadClass), alignof(GstGLBaseMixerPadClass));
    printf("%s;%zu;%zu\n", "GstGLBaseSrc", sizeof(GstGLBaseSrc), alignof(GstGLBaseSrc));
    printf("%s;%zu;%zu\n", "GstGLBaseSrcClass", sizeof(GstGLBaseSrcClass), alignof(GstGLBaseSrcClass));
    printf("%s;%zu;%zu\n", "GstGLBuffer", sizeof(GstGLBuffer), alignof(GstGLBuffer));
    printf("%s;%zu;%zu\n", "GstGLBufferAllocationParams", sizeof(GstGLBufferAllocationParams), alignof(GstGLBufferAllocationParams));
    printf("%s;%zu;%zu\n", "GstGLBufferAllocator", sizeof(GstGLBufferAllocator), alignof(GstGLBufferAllocator));
    printf("%s;%zu;%zu\n", "GstGLBufferAllocatorClass", sizeof(GstGLBufferAllocatorClass), alignof(GstGLBufferAllocatorClass));
    printf("%s;%zu;%zu\n", "GstGLBufferPool", sizeof(GstGLBufferPool), alignof(GstGLBufferPool));
    printf("%s;%zu;%zu\n", "GstGLBufferPoolClass", sizeof(GstGLBufferPoolClass), alignof(GstGLBufferPoolClass));
    printf("%s;%zu;%zu\n", "GstGLColorConvert", sizeof(GstGLColorConvert), alignof(GstGLColorConvert));
    printf("%s;%zu;%zu\n", "GstGLColorConvertClass", sizeof(GstGLColorConvertClass), alignof(GstGLColorConvertClass));
    printf("%s;%zu;%zu\n", "GstGLConfigCaveat", sizeof(GstGLConfigCaveat), alignof(GstGLConfigCaveat));
    printf("%s;%zu;%zu\n", "GstGLConfigSurfaceType", sizeof(GstGLConfigSurfaceType), alignof(GstGLConfigSurfaceType));
    printf("%s;%zu;%zu\n", "GstGLContext", sizeof(GstGLContext), alignof(GstGLContext));
    printf("%s;%zu;%zu\n", "GstGLContextClass", sizeof(GstGLContextClass), alignof(GstGLContextClass));
    printf("%s;%zu;%zu\n", "GstGLContextError", sizeof(GstGLContextError), alignof(GstGLContextError));
    printf("%s;%zu;%zu\n", "GstGLDisplay", sizeof(GstGLDisplay), alignof(GstGLDisplay));
    printf("%s;%zu;%zu\n", "GstGLDisplayClass", sizeof(GstGLDisplayClass), alignof(GstGLDisplayClass));
    printf("%s;%zu;%zu\n", "GstGLDisplayType", sizeof(GstGLDisplayType), alignof(GstGLDisplayType));
    printf("%s;%zu;%zu\n", "GstGLFilter", sizeof(GstGLFilter), alignof(GstGLFilter));
    printf("%s;%zu;%zu\n", "GstGLFilterClass", sizeof(GstGLFilterClass), alignof(GstGLFilterClass));
    printf("%s;%zu;%zu\n", "GstGLFormat", sizeof(GstGLFormat), alignof(GstGLFormat));
    printf("%s;%zu;%zu\n", "GstGLFramebuffer", sizeof(GstGLFramebuffer), alignof(GstGLFramebuffer));
    printf("%s;%zu;%zu\n", "GstGLFramebufferClass", sizeof(GstGLFramebufferClass), alignof(GstGLFramebufferClass));
    printf("%s;%zu;%zu\n", "GstGLMemory", sizeof(GstGLMemory), alignof(GstGLMemory));
    printf("%s;%zu;%zu\n", "GstGLMemoryAllocator", sizeof(GstGLMemoryAllocator), alignof(GstGLMemoryAllocator));
    printf("%s;%zu;%zu\n", "GstGLMemoryAllocatorClass", sizeof(GstGLMemoryAllocatorClass), alignof(GstGLMemoryAllocatorClass));
    printf("%s;%zu;%zu\n", "GstGLMemoryPBO", sizeof(GstGLMemoryPBO), alignof(GstGLMemoryPBO));
    printf("%s;%zu;%zu\n", "GstGLMemoryPBOAllocator", sizeof(GstGLMemoryPBOAllocator), alignof(GstGLMemoryPBOAllocator));
    printf("%s;%zu;%zu\n", "GstGLMemoryPBOAllocatorClass", sizeof(GstGLMemoryPBOAllocatorClass), alignof(GstGLMemoryPBOAllocatorClass));
    printf("%s;%zu;%zu\n", "GstGLMixer", sizeof(GstGLMixer), alignof(GstGLMixer));
    printf("%s;%zu;%zu\n", "GstGLMixerClass", sizeof(GstGLMixerClass), alignof(GstGLMixerClass));
    printf("%s;%zu;%zu\n", "GstGLMixerPad", sizeof(GstGLMixerPad), alignof(GstGLMixerPad));
    printf("%s;%zu;%zu\n", "GstGLMixerPadClass", sizeof(GstGLMixerPadClass), alignof(GstGLMixerPadClass));
    printf("%s;%zu;%zu\n", "GstGLOverlayCompositor", sizeof(GstGLOverlayCompositor), alignof(GstGLOverlayCompositor));
    printf("%s;%zu;%zu\n", "GstGLOverlayCompositorClass", sizeof(GstGLOverlayCompositorClass), alignof(GstGLOverlayCompositorClass));
    printf("%s;%zu;%zu\n", "GstGLPlatform", sizeof(GstGLPlatform), alignof(GstGLPlatform));
    printf("%s;%zu;%zu\n", "GstGLQuery", sizeof(GstGLQuery), alignof(GstGLQuery));
    printf("%s;%zu;%zu\n", "GstGLQueryType", sizeof(GstGLQueryType), alignof(GstGLQueryType));
    printf("%s;%zu;%zu\n", "GstGLRenderbuffer", sizeof(GstGLRenderbuffer), alignof(GstGLRenderbuffer));
    printf("%s;%zu;%zu\n", "GstGLRenderbufferAllocationParams", sizeof(GstGLRenderbufferAllocationParams), alignof(GstGLRenderbufferAllocationParams));
    printf("%s;%zu;%zu\n", "GstGLRenderbufferAllocator", sizeof(GstGLRenderbufferAllocator), alignof(GstGLRenderbufferAllocator));
    printf("%s;%zu;%zu\n", "GstGLRenderbufferAllocatorClass", sizeof(GstGLRenderbufferAllocatorClass), alignof(GstGLRenderbufferAllocatorClass));
    printf("%s;%zu;%zu\n", "GstGLSLError", sizeof(GstGLSLError), alignof(GstGLSLError));
    printf("%s;%zu;%zu\n", "GstGLSLProfile", sizeof(GstGLSLProfile), alignof(GstGLSLProfile));
    printf("%s;%zu;%zu\n", "GstGLSLStage", sizeof(GstGLSLStage), alignof(GstGLSLStage));
    printf("%s;%zu;%zu\n", "GstGLSLStageClass", sizeof(GstGLSLStageClass), alignof(GstGLSLStageClass));
    printf("%s;%zu;%zu\n", "GstGLSLVersion", sizeof(GstGLSLVersion), alignof(GstGLSLVersion));
    printf("%s;%zu;%zu\n", "GstGLShader", sizeof(GstGLShader), alignof(GstGLShader));
    printf("%s;%zu;%zu\n", "GstGLShaderClass", sizeof(GstGLShaderClass), alignof(GstGLShaderClass));
    printf("%s;%zu;%zu\n", "GstGLStereoDownmix", sizeof(GstGLStereoDownmix), alignof(GstGLStereoDownmix));
    printf("%s;%zu;%zu\n", "GstGLSyncMeta", sizeof(GstGLSyncMeta), alignof(GstGLSyncMeta));
    printf("%s;%zu;%zu\n", "GstGLTextureTarget", sizeof(GstGLTextureTarget), alignof(GstGLTextureTarget));
    printf("%s;%zu;%zu\n", "GstGLUpload", sizeof(GstGLUpload), alignof(GstGLUpload));
    printf("%s;%zu;%zu\n", "GstGLUploadClass", sizeof(GstGLUploadClass), alignof(GstGLUploadClass));
    printf("%s;%zu;%zu\n", "GstGLUploadReturn", sizeof(GstGLUploadReturn), alignof(GstGLUploadReturn));
    printf("%s;%zu;%zu\n", "GstGLVideoAllocationParams", sizeof(GstGLVideoAllocationParams), alignof(GstGLVideoAllocationParams));
    printf("%s;%zu;%zu\n", "GstGLViewConvert", sizeof(GstGLViewConvert), alignof(GstGLViewConvert));
    printf("%s;%zu;%zu\n", "GstGLViewConvertClass", sizeof(GstGLViewConvertClass), alignof(GstGLViewConvertClass));
    printf("%s;%zu;%zu\n", "GstGLWindow", sizeof(GstGLWindow), alignof(GstGLWindow));
    printf("%s;%zu;%zu\n", "GstGLWindowClass", sizeof(GstGLWindowClass), alignof(GstGLWindowClass));
    printf("%s;%zu;%zu\n", "GstGLWindowError", sizeof(GstGLWindowError), alignof(GstGLWindowError));
    return 0;
}
