// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

use crate::ffi;

glib::wrapper! {
    #[doc(alias = "GstVideoFilter")]
    pub struct VideoFilter(Object<ffi::GstVideoFilter, ffi::GstVideoFilterClass>) @extends gst_base::BaseTransform, gst::Element, gst::Object;

    match fn {
        type_ => || ffi::gst_video_filter_get_type(),
    }
}

impl VideoFilter {
    pub const NONE: Option<&'static VideoFilter> = None;
}

unsafe impl Send for VideoFilter {}
unsafe impl Sync for VideoFilter {}
