// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::{ffi, Accessible};
use glib::prelude::*;

glib::wrapper! {
    #[doc(alias = "GtkAccessibleRange")]
    pub struct AccessibleRange(Interface<ffi::GtkAccessibleRange, ffi::GtkAccessibleRangeInterface>) @requires Accessible;

    match fn {
        type_ => || ffi::gtk_accessible_range_get_type(),
    }
}

impl AccessibleRange {
    pub const NONE: Option<&'static AccessibleRange> = None;
}

mod sealed {
    pub trait Sealed {}
    impl<T: super::IsA<super::AccessibleRange>> Sealed for T {}
}

pub trait AccessibleRangeExt: IsA<AccessibleRange> + sealed::Sealed + 'static {}

impl<O: IsA<AccessibleRange>> AccessibleRangeExt for O {}
