// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

use crate::ffi;
use glib::prelude::*;

glib::wrapper! {
    #[doc(alias = "GstPhysMemoryAllocator")]
    pub struct PhysMemoryAllocator(Interface<ffi::GstPhysMemoryAllocator, ffi::GstPhysMemoryAllocatorInterface>) @requires gst::Allocator;

    match fn {
        type_ => || ffi::gst_phys_memory_allocator_get_type(),
    }
}

impl PhysMemoryAllocator {
    pub const NONE: Option<&'static PhysMemoryAllocator> = None;
}

unsafe impl Send for PhysMemoryAllocator {}
unsafe impl Sync for PhysMemoryAllocator {}

mod sealed {
    pub trait Sealed {}
    impl<T: super::IsA<super::PhysMemoryAllocator>> Sealed for T {}
}

pub trait PhysMemoryAllocatorExt: IsA<PhysMemoryAllocator> + sealed::Sealed + 'static {}

impl<O: IsA<PhysMemoryAllocator>> PhysMemoryAllocatorExt for O {}
