// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::{ffi, Device, DevicePadFeature};
use glib::{prelude::*, translate::*};

glib::wrapper! {
    #[doc(alias = "GdkDevicePad")]
    pub struct DevicePad(Interface<ffi::GdkDevicePad, ffi::GdkDevicePadInterface>) @requires Device;

    match fn {
        type_ => || ffi::gdk_device_pad_get_type(),
    }
}

impl DevicePad {
    pub const NONE: Option<&'static DevicePad> = None;
}

mod sealed {
    pub trait Sealed {}
    impl<T: super::IsA<super::DevicePad>> Sealed for T {}
}

pub trait DevicePadExt: IsA<DevicePad> + sealed::Sealed + 'static {
    #[doc(alias = "gdk_device_pad_get_feature_group")]
    #[doc(alias = "get_feature_group")]
    fn feature_group(&self, feature: DevicePadFeature, feature_idx: i32) -> i32 {
        unsafe {
            ffi::gdk_device_pad_get_feature_group(
                self.as_ref().to_glib_none().0,
                feature.into_glib(),
                feature_idx,
            )
        }
    }

    #[doc(alias = "gdk_device_pad_get_group_n_modes")]
    #[doc(alias = "get_group_n_modes")]
    fn group_n_modes(&self, group_idx: i32) -> i32 {
        unsafe { ffi::gdk_device_pad_get_group_n_modes(self.as_ref().to_glib_none().0, group_idx) }
    }

    #[doc(alias = "gdk_device_pad_get_n_features")]
    #[doc(alias = "get_n_features")]
    fn n_features(&self, feature: DevicePadFeature) -> i32 {
        unsafe {
            ffi::gdk_device_pad_get_n_features(self.as_ref().to_glib_none().0, feature.into_glib())
        }
    }

    #[doc(alias = "gdk_device_pad_get_n_groups")]
    #[doc(alias = "get_n_groups")]
    fn n_groups(&self) -> i32 {
        unsafe { ffi::gdk_device_pad_get_n_groups(self.as_ref().to_glib_none().0) }
    }
}

impl<O: IsA<DevicePad>> DevicePadExt for O {}
