// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::ffi;

glib::wrapper! {
    #[derive(Debug)]
    pub struct UnixMountEntry(Boxed<ffi::GUnixMountEntry>);

    match fn {
        copy => |ptr| glib::gobject_ffi::g_boxed_copy(ffi::g_unix_mount_entry_get_type(), ptr as *mut _) as *mut ffi::GUnixMountEntry,
        free => |ptr| glib::gobject_ffi::g_boxed_free(ffi::g_unix_mount_entry_get_type(), ptr as *mut _),
        type_ => || ffi::g_unix_mount_entry_get_type(),
    }
}

unsafe impl Send for UnixMountEntry {}
unsafe impl Sync for UnixMountEntry {}
