// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

use crate::{ffi, FdAllocator};

glib::wrapper! {
    #[doc(alias = "GstShmAllocator")]
    pub struct ShmAllocator(Object<ffi::GstShmAllocator, ffi::GstShmAllocatorClass>) @extends FdAllocator, gst::Allocator;

    match fn {
        type_ => || ffi::gst_shm_allocator_get_type(),
    }
}

impl ShmAllocator {}

unsafe impl Send for ShmAllocator {}
unsafe impl Sync for ShmAllocator {}
