// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

mod pixbuf;
pub use self::pixbuf::Pixbuf;

mod pixbuf_animation;
pub use self::pixbuf_animation::PixbufAnimation;

mod pixbuf_loader;
pub use self::pixbuf_loader::PixbufLoader;

mod pixbuf_non_anim;
pub use self::pixbuf_non_anim::PixbufNonAnim;

mod pixbuf_simple_anim;
pub use self::pixbuf_simple_anim::PixbufSimpleAnim;

mod pixbuf_format;
pub use self::pixbuf_format::PixbufFormat;

mod enums;
pub use self::enums::Colorspace;
pub use self::enums::InterpType;
#[allow(deprecated)]
pub use self::enums::PixbufAlphaMode;
pub use self::enums::PixbufError;
pub use self::enums::PixbufRotation;

mod flags;
pub use self::flags::PixbufFormatFlags;

pub(crate) mod traits {
    pub use super::pixbuf_animation::PixbufAnimationExt;
    pub use super::pixbuf_loader::PixbufLoaderExt;
}
