// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

#include "manual.h"
#include <stdalign.h>
#include <stdio.h>

int main() {
    printf("%s;%zu;%zu\n", "GstAllocationParams", sizeof(GstAllocationParams), alignof(GstAllocationParams));
    printf("%s;%zu;%zu\n", "GstAllocator", sizeof(GstAllocator), alignof(GstAllocator));
    printf("%s;%zu;%zu\n", "GstAllocatorClass", sizeof(GstAllocatorClass), alignof(GstAllocatorClass));
    printf("%s;%zu;%zu\n", "GstAllocatorFlags", sizeof(GstAllocatorFlags), alignof(GstAllocatorFlags));
    printf("%s;%zu;%zu\n", "GstBin", sizeof(GstBin), alignof(GstBin));
    printf("%s;%zu;%zu\n", "GstBinClass", sizeof(GstBinClass), alignof(GstBinClass));
    printf("%s;%zu;%zu\n", "GstBinFlags", sizeof(GstBinFlags), alignof(GstBinFlags));
    printf("%s;%zu;%zu\n", "GstBuffer", sizeof(GstBuffer), alignof(GstBuffer));
    printf("%s;%zu;%zu\n", "GstBufferCopyFlags", sizeof(GstBufferCopyFlags), alignof(GstBufferCopyFlags));
    printf("%s;%zu;%zu\n", "GstBufferFlags", sizeof(GstBufferFlags), alignof(GstBufferFlags));
    printf("%s;%zu;%zu\n", "GstBufferMapInfo", sizeof(GstBufferMapInfo), alignof(GstBufferMapInfo));
    printf("%s;%zu;%zu\n", "GstBufferPool", sizeof(GstBufferPool), alignof(GstBufferPool));
    printf("%s;%zu;%zu\n", "GstBufferPoolAcquireFlags", sizeof(GstBufferPoolAcquireFlags), alignof(GstBufferPoolAcquireFlags));
    printf("%s;%zu;%zu\n", "GstBufferPoolAcquireParams", sizeof(GstBufferPoolAcquireParams), alignof(GstBufferPoolAcquireParams));
    printf("%s;%zu;%zu\n", "GstBufferPoolClass", sizeof(GstBufferPoolClass), alignof(GstBufferPoolClass));
    printf("%s;%zu;%zu\n", "GstBufferingMode", sizeof(GstBufferingMode), alignof(GstBufferingMode));
    printf("%s;%zu;%zu\n", "GstBus", sizeof(GstBus), alignof(GstBus));
    printf("%s;%zu;%zu\n", "GstBusClass", sizeof(GstBusClass), alignof(GstBusClass));
    printf("%s;%zu;%zu\n", "GstBusFlags", sizeof(GstBusFlags), alignof(GstBusFlags));
    printf("%s;%zu;%zu\n", "GstBusSyncReply", sizeof(GstBusSyncReply), alignof(GstBusSyncReply));
    printf("%s;%zu;%zu\n", "GstByteArrayInterface", sizeof(GstByteArrayInterface), alignof(GstByteArrayInterface));
    printf("%s;%zu;%zu\n", "GstCaps", sizeof(GstCaps), alignof(GstCaps));
    printf("%s;%zu;%zu\n", "GstCapsFlags", sizeof(GstCapsFlags), alignof(GstCapsFlags));
    printf("%s;%zu;%zu\n", "GstCapsIntersectMode", sizeof(GstCapsIntersectMode), alignof(GstCapsIntersectMode));
    printf("%s;%zu;%zu\n", "GstChildProxyInterface", sizeof(GstChildProxyInterface), alignof(GstChildProxyInterface));
    printf("%s;%zu;%zu\n", "GstClock", sizeof(GstClock), alignof(GstClock));
    printf("%s;%zu;%zu\n", "GstClockClass", sizeof(GstClockClass), alignof(GstClockClass));
    printf("%s;%zu;%zu\n", "GstClockEntry", sizeof(GstClockEntry), alignof(GstClockEntry));
    printf("%s;%zu;%zu\n", "GstClockEntryType", sizeof(GstClockEntryType), alignof(GstClockEntryType));
    printf("%s;%zu;%zu\n", "GstClockFlags", sizeof(GstClockFlags), alignof(GstClockFlags));
    printf("%s;%zu;%zu\n", "GstClockID", sizeof(GstClockID), alignof(GstClockID));
    printf("%s;%zu;%zu\n", "GstClockReturn", sizeof(GstClockReturn), alignof(GstClockReturn));
    printf("%s;%zu;%zu\n", "GstClockTime", sizeof(GstClockTime), alignof(GstClockTime));
    printf("%s;%zu;%zu\n", "GstClockTimeDiff", sizeof(GstClockTimeDiff), alignof(GstClockTimeDiff));
    printf("%s;%zu;%zu\n", "GstClockType", sizeof(GstClockType), alignof(GstClockType));
    printf("%s;%zu;%zu\n", "GstControlBinding", sizeof(GstControlBinding), alignof(GstControlBinding));
    printf("%s;%zu;%zu\n", "GstControlBindingClass", sizeof(GstControlBindingClass), alignof(GstControlBindingClass));
    printf("%s;%zu;%zu\n", "GstControlSource", sizeof(GstControlSource), alignof(GstControlSource));
    printf("%s;%zu;%zu\n", "GstControlSourceClass", sizeof(GstControlSourceClass), alignof(GstControlSourceClass));
    printf("%s;%zu;%zu\n", "GstCoreError", sizeof(GstCoreError), alignof(GstCoreError));
    printf("%s;%zu;%zu\n", "GstCustomMeta", sizeof(GstCustomMeta), alignof(GstCustomMeta));
    printf("%s;%zu;%zu\n", "GstDebugCategory", sizeof(GstDebugCategory), alignof(GstDebugCategory));
    printf("%s;%zu;%zu\n", "GstDebugColorFlags", sizeof(GstDebugColorFlags), alignof(GstDebugColorFlags));
    printf("%s;%zu;%zu\n", "GstDebugColorMode", sizeof(GstDebugColorMode), alignof(GstDebugColorMode));
    printf("%s;%zu;%zu\n", "GstDebugGraphDetails", sizeof(GstDebugGraphDetails), alignof(GstDebugGraphDetails));
    printf("%s;%zu;%zu\n", "GstDebugLevel", sizeof(GstDebugLevel), alignof(GstDebugLevel));
    printf("%s;%zu;%zu\n", "GstDevice", sizeof(GstDevice), alignof(GstDevice));
    printf("%s;%zu;%zu\n", "GstDeviceClass", sizeof(GstDeviceClass), alignof(GstDeviceClass));
    printf("%s;%zu;%zu\n", "GstDeviceMonitor", sizeof(GstDeviceMonitor), alignof(GstDeviceMonitor));
    printf("%s;%zu;%zu\n", "GstDeviceMonitorClass", sizeof(GstDeviceMonitorClass), alignof(GstDeviceMonitorClass));
    printf("%s;%zu;%zu\n", "GstDeviceProvider", sizeof(GstDeviceProvider), alignof(GstDeviceProvider));
    printf("%s;%zu;%zu\n", "GstDeviceProviderClass", sizeof(GstDeviceProviderClass), alignof(GstDeviceProviderClass));
    printf("%s;%zu;%zu\n", "GstElement", sizeof(GstElement), alignof(GstElement));
    printf("%s;%zu;%zu\n", "GstElementClass", sizeof(GstElementClass), alignof(GstElementClass));
    printf("%s;%zu;%zu\n", "GstElementFactoryListType", sizeof(GstElementFactoryListType), alignof(GstElementFactoryListType));
    printf("%s;%zu;%zu\n", "GstElementFlags", sizeof(GstElementFlags), alignof(GstElementFlags));
    printf("%s;%zu;%zu\n", "GstEvent", sizeof(GstEvent), alignof(GstEvent));
    printf("%s;%zu;%zu\n", "GstEventType", sizeof(GstEventType), alignof(GstEventType));
    printf("%s;%zu;%zu\n", "GstEventTypeFlags", sizeof(GstEventTypeFlags), alignof(GstEventTypeFlags));
    printf("%s;%zu;%zu\n", "GstFlowReturn", sizeof(GstFlowReturn), alignof(GstFlowReturn));
    printf("%s;%zu;%zu\n", "GstFormat", sizeof(GstFormat), alignof(GstFormat));
    printf("%s;%zu;%zu\n", "GstFormatDefinition", sizeof(GstFormatDefinition), alignof(GstFormatDefinition));
    printf("%s;%zu;%zu\n", "GstGapFlags", sizeof(GstGapFlags), alignof(GstGapFlags));
    printf("%s;%zu;%zu\n", "GstGhostPad", sizeof(GstGhostPad), alignof(GstGhostPad));
    printf("%s;%zu;%zu\n", "GstGhostPadClass", sizeof(GstGhostPadClass), alignof(GstGhostPadClass));
    printf("%s;%zu;%zu\n", "GstIterator", sizeof(GstIterator), alignof(GstIterator));
    printf("%s;%zu;%zu\n", "GstIteratorItem", sizeof(GstIteratorItem), alignof(GstIteratorItem));
    printf("%s;%zu;%zu\n", "GstIteratorResult", sizeof(GstIteratorResult), alignof(GstIteratorResult));
    printf("%s;%zu;%zu\n", "GstLibraryError", sizeof(GstLibraryError), alignof(GstLibraryError));
    printf("%s;%zu;%zu\n", "GstLockFlags", sizeof(GstLockFlags), alignof(GstLockFlags));
    printf("%s;%zu;%zu\n", "GstMapFlags", sizeof(GstMapFlags), alignof(GstMapFlags));
    printf("%s;%zu;%zu\n", "GstMapInfo", sizeof(GstMapInfo), alignof(GstMapInfo));
    printf("%s;%zu;%zu\n", "GstMemory", sizeof(GstMemory), alignof(GstMemory));
    printf("%s;%zu;%zu\n", "GstMemoryFlags", sizeof(GstMemoryFlags), alignof(GstMemoryFlags));
    printf("%s;%zu;%zu\n", "GstMemoryMapInfo", sizeof(GstMemoryMapInfo), alignof(GstMemoryMapInfo));
    printf("%s;%zu;%zu\n", "GstMessage", sizeof(GstMessage), alignof(GstMessage));
    printf("%s;%zu;%zu\n", "GstMessageType", sizeof(GstMessageType), alignof(GstMessageType));
    printf("%s;%zu;%zu\n", "GstMeta", sizeof(GstMeta), alignof(GstMeta));
    printf("%s;%zu;%zu\n", "GstMetaFlags", sizeof(GstMetaFlags), alignof(GstMetaFlags));
    printf("%s;%zu;%zu\n", "GstMetaInfo", sizeof(GstMetaInfo), alignof(GstMetaInfo));
    printf("%s;%zu;%zu\n", "GstMetaTransformCopy", sizeof(GstMetaTransformCopy), alignof(GstMetaTransformCopy));
    printf("%s;%zu;%zu\n", "GstMiniObject", sizeof(GstMiniObject), alignof(GstMiniObject));
    printf("%s;%zu;%zu\n", "GstMiniObjectFlags", sizeof(GstMiniObjectFlags), alignof(GstMiniObjectFlags));
    printf("%s;%zu;%zu\n", "GstObject", sizeof(GstObject), alignof(GstObject));
    printf("%s;%zu;%zu\n", "GstObjectClass", sizeof(GstObjectClass), alignof(GstObjectClass));
    printf("%s;%zu;%zu\n", "GstObjectFlags", sizeof(GstObjectFlags), alignof(GstObjectFlags));
    printf("%s;%zu;%zu\n", "GstPad", sizeof(GstPad), alignof(GstPad));
    printf("%s;%zu;%zu\n", "GstPadClass", sizeof(GstPadClass), alignof(GstPadClass));
    printf("%s;%zu;%zu\n", "GstPadDirection", sizeof(GstPadDirection), alignof(GstPadDirection));
    printf("%s;%zu;%zu\n", "GstPadFlags", sizeof(GstPadFlags), alignof(GstPadFlags));
    printf("%s;%zu;%zu\n", "GstPadLinkCheck", sizeof(GstPadLinkCheck), alignof(GstPadLinkCheck));
    printf("%s;%zu;%zu\n", "GstPadLinkReturn", sizeof(GstPadLinkReturn), alignof(GstPadLinkReturn));
    printf("%s;%zu;%zu\n", "GstPadMode", sizeof(GstPadMode), alignof(GstPadMode));
    printf("%s;%zu;%zu\n", "GstPadPresence", sizeof(GstPadPresence), alignof(GstPadPresence));
    printf("%s;%zu;%zu\n", "GstPadProbeInfo", sizeof(GstPadProbeInfo), alignof(GstPadProbeInfo));
    printf("%s;%zu;%zu\n", "GstPadProbeReturn", sizeof(GstPadProbeReturn), alignof(GstPadProbeReturn));
    printf("%s;%zu;%zu\n", "GstPadProbeType", sizeof(GstPadProbeType), alignof(GstPadProbeType));
    printf("%s;%zu;%zu\n", "GstPadTemplate", sizeof(GstPadTemplate), alignof(GstPadTemplate));
    printf("%s;%zu;%zu\n", "GstPadTemplateClass", sizeof(GstPadTemplateClass), alignof(GstPadTemplateClass));
    printf("%s;%zu;%zu\n", "GstPadTemplateFlags", sizeof(GstPadTemplateFlags), alignof(GstPadTemplateFlags));
    printf("%s;%zu;%zu\n", "GstParamSpecArray", sizeof(GstParamSpecArray), alignof(GstParamSpecArray));
    printf("%s;%zu;%zu\n", "GstParamSpecFraction", sizeof(GstParamSpecFraction), alignof(GstParamSpecFraction));
    printf("%s;%zu;%zu\n", "GstParentBufferMeta", sizeof(GstParentBufferMeta), alignof(GstParentBufferMeta));
    printf("%s;%zu;%zu\n", "GstParseError", sizeof(GstParseError), alignof(GstParseError));
    printf("%s;%zu;%zu\n", "GstParseFlags", sizeof(GstParseFlags), alignof(GstParseFlags));
    printf("%s;%zu;%zu\n", "GstPipeline", sizeof(GstPipeline), alignof(GstPipeline));
    printf("%s;%zu;%zu\n", "GstPipelineClass", sizeof(GstPipelineClass), alignof(GstPipelineClass));
    printf("%s;%zu;%zu\n", "GstPipelineFlags", sizeof(GstPipelineFlags), alignof(GstPipelineFlags));
    printf("%s;%zu;%zu\n", "GstPluginAPIFlags", sizeof(GstPluginAPIFlags), alignof(GstPluginAPIFlags));
    printf("%s;%zu;%zu\n", "GstPluginDependencyFlags", sizeof(GstPluginDependencyFlags), alignof(GstPluginDependencyFlags));
    printf("%s;%zu;%zu\n", "GstPluginDesc", sizeof(GstPluginDesc), alignof(GstPluginDesc));
    printf("%s;%zu;%zu\n", "GstPluginError", sizeof(GstPluginError), alignof(GstPluginError));
    printf("%s;%zu;%zu\n", "GstPluginFlags", sizeof(GstPluginFlags), alignof(GstPluginFlags));
    printf("%s;%zu;%zu\n", "GstPollFD", sizeof(GstPollFD), alignof(GstPollFD));
    printf("%s;%zu;%zu\n", "GstPresetInterface", sizeof(GstPresetInterface), alignof(GstPresetInterface));
    printf("%s;%zu;%zu\n", "GstProgressType", sizeof(GstProgressType), alignof(GstProgressType));
    printf("%s;%zu;%zu\n", "GstPromise", sizeof(GstPromise), alignof(GstPromise));
    printf("%s;%zu;%zu\n", "GstPromiseResult", sizeof(GstPromiseResult), alignof(GstPromiseResult));
    printf("%s;%zu;%zu\n", "GstProtectionMeta", sizeof(GstProtectionMeta), alignof(GstProtectionMeta));
    printf("%s;%zu;%zu\n", "GstProxyPad", sizeof(GstProxyPad), alignof(GstProxyPad));
    printf("%s;%zu;%zu\n", "GstProxyPadClass", sizeof(GstProxyPadClass), alignof(GstProxyPadClass));
    printf("%s;%zu;%zu\n", "GstQOSType", sizeof(GstQOSType), alignof(GstQOSType));
    printf("%s;%zu;%zu\n", "GstQuery", sizeof(GstQuery), alignof(GstQuery));
    printf("%s;%zu;%zu\n", "GstQueryType", sizeof(GstQueryType), alignof(GstQueryType));
    printf("%s;%zu;%zu\n", "GstQueryTypeFlags", sizeof(GstQueryTypeFlags), alignof(GstQueryTypeFlags));
    printf("%s;%zu;%zu\n", "GstRank", sizeof(GstRank), alignof(GstRank));
    printf("%s;%zu;%zu\n", "GstReferenceTimestampMeta", sizeof(GstReferenceTimestampMeta), alignof(GstReferenceTimestampMeta));
    printf("%s;%zu;%zu\n", "GstRegistry", sizeof(GstRegistry), alignof(GstRegistry));
    printf("%s;%zu;%zu\n", "GstRegistryClass", sizeof(GstRegistryClass), alignof(GstRegistryClass));
    printf("%s;%zu;%zu\n", "GstResourceError", sizeof(GstResourceError), alignof(GstResourceError));
    printf("%s;%zu;%zu\n", "GstSchedulingFlags", sizeof(GstSchedulingFlags), alignof(GstSchedulingFlags));
    printf("%s;%zu;%zu\n", "GstSearchMode", sizeof(GstSearchMode), alignof(GstSearchMode));
    printf("%s;%zu;%zu\n", "GstSeekFlags", sizeof(GstSeekFlags), alignof(GstSeekFlags));
    printf("%s;%zu;%zu\n", "GstSeekType", sizeof(GstSeekType), alignof(GstSeekType));
    printf("%s;%zu;%zu\n", "GstSegment", sizeof(GstSegment), alignof(GstSegment));
    printf("%s;%zu;%zu\n", "GstSegmentFlags", sizeof(GstSegmentFlags), alignof(GstSegmentFlags));
    printf("%s;%zu;%zu\n", "GstSerializeFlags", sizeof(GstSerializeFlags), alignof(GstSerializeFlags));
    printf("%s;%zu;%zu\n", "GstSharedTaskPool", sizeof(GstSharedTaskPool), alignof(GstSharedTaskPool));
    printf("%s;%zu;%zu\n", "GstSharedTaskPoolClass", sizeof(GstSharedTaskPoolClass), alignof(GstSharedTaskPoolClass));
    printf("%s;%zu;%zu\n", "GstStackTraceFlags", sizeof(GstStackTraceFlags), alignof(GstStackTraceFlags));
    printf("%s;%zu;%zu\n", "GstState", sizeof(GstState), alignof(GstState));
    printf("%s;%zu;%zu\n", "GstStateChange", sizeof(GstStateChange), alignof(GstStateChange));
    printf("%s;%zu;%zu\n", "GstStateChangeReturn", sizeof(GstStateChangeReturn), alignof(GstStateChangeReturn));
    printf("%s;%zu;%zu\n", "GstStaticCaps", sizeof(GstStaticCaps), alignof(GstStaticCaps));
    printf("%s;%zu;%zu\n", "GstStaticPadTemplate", sizeof(GstStaticPadTemplate), alignof(GstStaticPadTemplate));
    printf("%s;%zu;%zu\n", "GstStream", sizeof(GstStream), alignof(GstStream));
    printf("%s;%zu;%zu\n", "GstStreamClass", sizeof(GstStreamClass), alignof(GstStreamClass));
    printf("%s;%zu;%zu\n", "GstStreamCollection", sizeof(GstStreamCollection), alignof(GstStreamCollection));
    printf("%s;%zu;%zu\n", "GstStreamCollectionClass", sizeof(GstStreamCollectionClass), alignof(GstStreamCollectionClass));
    printf("%s;%zu;%zu\n", "GstStreamError", sizeof(GstStreamError), alignof(GstStreamError));
    printf("%s;%zu;%zu\n", "GstStreamFlags", sizeof(GstStreamFlags), alignof(GstStreamFlags));
    printf("%s;%zu;%zu\n", "GstStreamStatusType", sizeof(GstStreamStatusType), alignof(GstStreamStatusType));
    printf("%s;%zu;%zu\n", "GstStreamType", sizeof(GstStreamType), alignof(GstStreamType));
    printf("%s;%zu;%zu\n", "GstStructure", sizeof(GstStructure), alignof(GstStructure));
    printf("%s;%zu;%zu\n", "GstStructureChangeType", sizeof(GstStructureChangeType), alignof(GstStructureChangeType));
    printf("%s;%zu;%zu\n", "GstSystemClock", sizeof(GstSystemClock), alignof(GstSystemClock));
    printf("%s;%zu;%zu\n", "GstSystemClockClass", sizeof(GstSystemClockClass), alignof(GstSystemClockClass));
    printf("%s;%zu;%zu\n", "GstTagFlag", sizeof(GstTagFlag), alignof(GstTagFlag));
    printf("%s;%zu;%zu\n", "GstTagList", sizeof(GstTagList), alignof(GstTagList));
    printf("%s;%zu;%zu\n", "GstTagMergeMode", sizeof(GstTagMergeMode), alignof(GstTagMergeMode));
    printf("%s;%zu;%zu\n", "GstTagScope", sizeof(GstTagScope), alignof(GstTagScope));
    printf("%s;%zu;%zu\n", "GstTagSetterInterface", sizeof(GstTagSetterInterface), alignof(GstTagSetterInterface));
    printf("%s;%zu;%zu\n", "GstTask", sizeof(GstTask), alignof(GstTask));
    printf("%s;%zu;%zu\n", "GstTaskClass", sizeof(GstTaskClass), alignof(GstTaskClass));
    printf("%s;%zu;%zu\n", "GstTaskPool", sizeof(GstTaskPool), alignof(GstTaskPool));
    printf("%s;%zu;%zu\n", "GstTaskPoolClass", sizeof(GstTaskPoolClass), alignof(GstTaskPoolClass));
    printf("%s;%zu;%zu\n", "GstTaskState", sizeof(GstTaskState), alignof(GstTaskState));
    printf("%s;%zu;%zu\n", "GstTimedValue", sizeof(GstTimedValue), alignof(GstTimedValue));
    printf("%s;%zu;%zu\n", "GstTocEntryType", sizeof(GstTocEntryType), alignof(GstTocEntryType));
    printf("%s;%zu;%zu\n", "GstTocLoopType", sizeof(GstTocLoopType), alignof(GstTocLoopType));
    printf("%s;%zu;%zu\n", "GstTocScope", sizeof(GstTocScope), alignof(GstTocScope));
    printf("%s;%zu;%zu\n", "GstTocSetterInterface", sizeof(GstTocSetterInterface), alignof(GstTocSetterInterface));
    printf("%s;%zu;%zu\n", "GstTracer", sizeof(GstTracer), alignof(GstTracer));
    printf("%s;%zu;%zu\n", "GstTracerClass", sizeof(GstTracerClass), alignof(GstTracerClass));
    printf("%s;%zu;%zu\n", "GstTracerValueFlags", sizeof(GstTracerValueFlags), alignof(GstTracerValueFlags));
    printf("%s;%zu;%zu\n", "GstTracerValueScope", sizeof(GstTracerValueScope), alignof(GstTracerValueScope));
    printf("%s;%zu;%zu\n", "GstTypeFind", sizeof(GstTypeFind), alignof(GstTypeFind));
    printf("%s;%zu;%zu\n", "GstTypeFindProbability", sizeof(GstTypeFindProbability), alignof(GstTypeFindProbability));
    printf("%s;%zu;%zu\n", "GstURIError", sizeof(GstURIError), alignof(GstURIError));
    printf("%s;%zu;%zu\n", "GstURIHandlerInterface", sizeof(GstURIHandlerInterface), alignof(GstURIHandlerInterface));
    printf("%s;%zu;%zu\n", "GstURIType", sizeof(GstURIType), alignof(GstURIType));
    printf("%s;%zu;%zu\n", "GstValueTable", sizeof(GstValueTable), alignof(GstValueTable));
    return 0;
}
