// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::ffi;

glib::wrapper! {
    #[doc(alias = "GdkWaylandGLContext")]
    pub struct WaylandGLContext(Object<ffi::GdkWaylandGLContext, ffi::GdkWaylandGLContextClass>) @extends gdk::GLContext, gdk::DrawContext;

    match fn {
        type_ => || ffi::gdk_wayland_gl_context_get_type(),
    }
}

impl WaylandGLContext {}
