// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::{ffi, Filter};

glib::wrapper! {
    #[doc(alias = "GtkCustomFilter")]
    pub struct CustomFilter(Object<ffi::GtkCustomFilter, ffi::GtkCustomFilterClass>) @extends Filter;

    match fn {
        type_ => || ffi::gtk_custom_filter_get_type(),
    }
}

impl CustomFilter {}
