// Take a look at the license at the top of the repository in the LICENSE file.

extern "C" {
    pub fn cairo_gobject_context_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_device_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_matrix_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_pattern_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_surface_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_rectangle_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_scaled_font_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_font_face_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_font_options_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_rectangle_int_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_region_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_status_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_content_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_operator_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_antialias_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_fill_rule_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_line_cap_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_line_join_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_text_cluster_flags_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_font_slant_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_font_weight_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_subpixel_order_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_hint_style_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_hint_metrics_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_font_type_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_path_data_type_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_device_type_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_surface_type_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_format_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_pattern_type_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_extend_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_filter_get_type() -> glib_sys::GType;
    pub fn cairo_gobject_region_overlap_get_type() -> glib_sys::GType;
}
