// Feel free to edit this file, it won't be regenerated by gir generator unless removed.

// The bindings include the pixbuf loader API
#define GDK_PIXBUF_ENABLE_BACKEND

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gdk-pixbuf/gdk-pixdata.h>
